$(document).ready(function() {
	
	// loading animation
	$("#routeselector").html('<div class="loading bar"><i class="sphere"></i></div>');
	$("#routeselector .loading.bar").attr("data-text","Loading map");

	// options
	var updateDelay = 10776;
	
	// setup map
	var zoom = 14;
	var center = [40.5, -74.45];
	var bounds = [
		[40.400, -74.65], // sw
		[40.600, -74.25], // ne
	];
	var mapDivID = "map";
	var mapOptions = {
		minZoom: 14,
		maxZoom: 18,
		zoomControl: false,
	};
	var boundsOptions = {
		'animate': false,
		'reset': true,
	};
	
	// fastclick
	$(function() {
		FastClick.attach(document.body);
	});
	
	// create map
	window.map = L.map(mapDivID, mapOptions).setView(center, zoom).setMaxBounds(bounds, boundsOptions);

	// setup and create tile layer on map
	var text = '<a href="https://ant.sr">ant.sr</a> | <div id="raptor"></div><br>'
		+ '<strong>Not affiliated with <a href="http://www.rutgers.edu">Rutgers</a>.</strong><br>'
		+ 'Bus data &copy; <a href="http://www.rutgers.edu/">Rutgers University</a><br>'
		+ 'Map data &copy; <a href="http://osm.org/copyright">OSM contributors</a>';
	
	L.tileLayer('http://{s}.tiles.antsar-static.com/{tileset}/{z}/{x}/{y}.png', {
		subdomains: ['a', 'b', 'c', 'd', 'e'],
		tileset: 'rutgers-black',
		errorTileUrl: 'http://tiles.antsar-static.com/generic/tile-blank-black.png',
		attribution: text,
	}).addTo(window.map);

	// init UI elements
	window.busMarkers = {};
	window.stopMarkers = {};
	window.polylines = [];
	
	// init data storage vars
	window.userLocation = false;
	
	// event listener for successful gps geolocation
	window.map.on('locationfound', function(e){
		var dotsize = 20;
		if (window.devicePixelRatio > 1) dotsize = 35; // retina
		showBlueDot(e.latlng, e.accuracy / 2, dotsize);
	});
	
	// event listener for gps geolocation failure
	window.map.on('locationerror', function(e){
		alert("Access to your current location was refused. To use the Current Location feature in the future, you may need to reset your browser's location warnings.");
	});
	
	// event listener for beginning of dragging.
	window.map.on('dragstart', function(e){
		// cancel geolocation on drag.
		if (window.userLocation == true) stopUserLocation();
	});
	
	// event listener for opening popup./
	/*
	window.map.on('popupopen', function(e){
		window.popup = e.popup.options;
		var zoom = window.map.getZoom;
		if (zoom < 16) zoom = 16
		//window.map.setView(e.popup._latlng, zoom);
	});
	*/
	
	
	// Bind user location getting to the user location getting button
	$("#locationButton").click(function() {
		if (window.userLocation == false) {
			startUserLocation();
		} else {
			stopUserLocation();
		}
	});	
	
	$("#routeselector .loading.bar").attr("data-text","Loading routes");
	
	// get routes and initialize route buttons. this is awkward; should refactor.
	getRouteConfig(function(routes, stops) {
		makeRouteButtons(routes);
	});
	
	// main app loop
	function mainLoop() {
		getRouteConfig(function(routes, stops) {
			updateMapBusStops(routes,stops);
			getVehicles(function(vehicles) {
				updateMapVehicles(routes, vehicles);
				removeStaleMarkers();
				updateRouteButtons(vehicles);
			});
		});
	}
	mainLoop();
	window.setInterval(mainLoop, updateDelay);
	
	$('#raptor').raptorize();
});

function startUserLocation() {
	$("#locationButton").css("background-image", "url(img/crosshair-blue.png)");
	window.map.locate({
		watch: true,
		setView: true,
		maxZoom: 18,
	});
	window.userLocation = true;
}

function stopUserLocation() {
	var blueDotFadeTimeSeconds = 15;
	window.map.stopLocate();
	window.userLocation = false;
	hideBlueDot(15);
	$("#locationButton").css("background-image", "url(img/crosshair.png)");
}

function showBlueDot(latlng, radius, size) {
	var markerIcon = L.icon({
		iconUrl: 'img/bluedot.png',
		iconSize: [size,size],
	});
	if (window.userLocationMarker == undefined) {
		window.userLocationMarker = L.marker(latlng, {icon: markerIcon}).addTo(map);
	} else {
		window.userLocationMarker.setLatLng(latlng);
	}

	if (radius < 25) radius = 1; // no tiny circles

	if (window.userLocationCircle == undefined) {
		window.userLocationCircle = L.circle(latlng, radius).addTo(map);
	} else {
		window.userLocationCircle.setLatLng(latlng).setRadius(radius);
	}
}

function hideBlueDot(userLocationFadeTime) {
	var fps = 30;
	var hideBlueDotFrames = userLocationFadeTime * fps;
	var delay = Math.round(1000 / fps);
	window.hideBlueDotTimers = [];
	var frame = 0;
	function drawFrames( frame, delay, totalFrames ) {
		var opacity = 1 - frame/totalFrames;
		window.userLocationMarker.setOpacity(opacity);
		if (totalFrames > frame) {
			window.hideBlueDotTimers.push(setTimeout(function() {
				drawFrames(frame+1, delay, totalFrames);
			}, delay));
		} else {
			if (window.userLocationMarker != undefined) {
				window.map.removeLayer(window.userLocationMarker);
				delete window.userLocationMarker;
			}
		}
	}
	if (window.userLocationCircle != undefined) window.map.removeLayer(window.userLocationCircle);
	delete window.userLocationCircle;
	if (userLocationMarker) drawFrames(frame, delay, hideBlueDotFrames);
}

function getRouteConfig(grc_callback) {
	$.ajax({
		url: "ajax.php?command=routeConfig&a=rutgers",
		type: 'GET',
		success: function(data) {
			if (grc_callback != undefined) grc_callback(data['routes'], data['stops']);
		},
		fail: function() {
			alert("Could not get routes from NextBus. Please try again later.");
		}
	});
}

function getVehicles(gv_callback) {	
	var time = 0;
	if (window.getVehicles_lastTime != undefined) time = window.getVehicles_lastTime;
	$.ajax({
		url: "ajax.php?command=vehicleLocations&a=rutgers&t=" + time,
		type: 'GET',
		success: function(data) {
			window.getVehicles_lastTime = data['lastTime'];
			// window.vehicles = data['vehicles'];
			if (gv_callback != undefined) gv_callback(data['vehicles']);
		},
		fail: function() {
			alert("Could not get vehicle locations from NextBus. Please try again later.");
		}
	});
}

function updateMapBusStops(routesIn,stops,maxPredictions) {	
	if (window.map != undefined && stops != undefined) {
		for (i in stops) {
			if (stops[i]['lat'] != undefined && stops[i]['lon'] != undefined) {
				var markerIcon = L.icon({
					iconUrl: 'img/star.png',
					iconSize: [25,24],
				});
				var routes = [];
				for (j in stops[i]['routes']) {
					var string = "<strong>"+routesIn[j]['title']+"</strong>" + ": ";
					for (k in stops[i]['routes'][j]) {
						var sec = Math.floor((stops[i]['routes'][j][k]['epochTime'] - (new Date).getTime()) / 1000);
						var min = Math.floor(sec / 60);
						var sec = sec % 60;
						string += '<a class="tooltip prediction" title="Bus #'+stops[i]['routes'][j][k]['vehicle']+'">' + min + "</a>, ";
					}
					if (string != "<strong>"+routesIn[j]['title']+"</strong>" + ": ") {
						string = string.substring(0, string.length-2);
						routes.push(string);
					}
				}
				if (routes.length == 0) var routeList = "<div class='centered em'>no predictions</div>";
				else var routeList = routes.sort().join("<br>")
				var popup = '<div class="title">'+stops[i]['title']+'</div>' + routeList;
				if (window.stopMarkers[stops[i]['tag']] == undefined) {
					var markerOpts = {
						icon: markerIcon,
						title: stops[i]['title'],
						opacity: 1,
						riseOnHover: true,
						zIndexOffset: -1000,
						riseOffset: 1500,
					};
					var popupOpts = {
						closeButton: false,
						keepInView: true,
					};
					window.stopMarkers[stops[i]['tag']] = L.marker([stops[i]['lat'], stops[i]['lon']], markerOpts).addTo(window.map).bindPopup(popup, popupOpts);
				} else {
					window.stopMarkers[stops[i]['tag']].setLatLng([stops[i]['lat'], stops[i]['lon']]).setPopupContent(popup);
				}
			}
		}
	}
}

function makeRouteButtons(routes) {
	if (window.map != undefined && routes != undefined) {
		$("#routeselector").append('<button class="routeBtn routeBtn-=allRoutes" data-selected="false" data-route="allroutes">All</button>');
		for (i in routes) {
			$("#routeselector").append('<button class="routeBtn" data-selected="true" data-route="'+i+'">'+routes[i]['title']+'</button>');
		}
		$("#routeselector .routeBtn").hide();
		$('#routeselector').on("click", ".routeBtn", function(e) {
			if ($(this).attr('data-route') == "allroutes") {
				$(this).siblings().click();
			} else {
				if ($(this).attr('data-selected') == 'true') {
					// hide this route
					for (i in window.busMarkers[$(this).attr('data-route')]) {
						window.busMarkers[$(this).attr('data-route')][i].setOpacity(0);
					}
					$(this).attr('data-selected', 'false');
				} else {
					// show this route
					for (i in window.busMarkers[$(this).attr('data-route')]) {
						window.busMarkers[$(this).attr('data-route')][i].setOpacity(1);
					}
					$(this).attr('data-selected', 'true');
				}
			}
		});
	}
}

function updateRouteButtons(vehicles) {
	if (window.map != undefined) {
		visibleRoutes = {};
		for (i in window.busMarkers) {
			for (j in window.busMarkers[i]) {
				visibleRoutes[i] = true;
			}
		}
		if ($("#routeselector .loading").length > 0) $("#routeselector .loading").remove();
		$("#routeselector .routeBtn").not("[data-route='allroutes']").each(function() {
			if (visibleRoutes[$(this).attr('data-route')] == undefined && $(this).is(":visible") == true) $(this).hide();
			else if (visibleRoutes[$(this).attr('data-route')] == true && $(this).is(":visible") == false) $(this).show();
		});
		if ($("#routeselector .routeBtn:visible").length > 0 && $("#routeselector .routeBtn[data-route='allroutes']").is(":visible") == false) {
			$("#routeselector .routeBtn[data-route='allroutes']").show();
		}
	}
}

function updateMapVehicles(routes,vehicles) {
	if (window.map != undefined) {
		for (i in vehicles) {
			
			if (window.busMarkers[vehicles[i]['routeTag']] == undefined) {
				window.busMarkers[vehicles[i]['routeTag']] = {};
			}

			var stops = [];
			for (j in vehicles[i]['stops']) {
				var sec = Math.floor((vehicles[i]['stops'][j]['epochTime'] - (new Date).getTime()) / 1000);
				var min = Math.floor(sec / 60);
				var sec = sec % 60;
				stops.push("<span class='prediction'>" + min + '</span> ' + vehicles[i]['stops'][j]['title'] );
			}
			var stopList = stops.join("<br>")
			label = vehicles[i]['route'];
			popup = "";
			popup += "<div class='title'>" + label + "</div>";
			popup += stopList;
			popup += "<div class='footer'>Bus #" + vehicles[i]['id'] + "</div>";
			popup += "";
			if (window.busMarkers[vehicles[i]['routeTag']][i] == undefined) {

				window.busMarkers[vehicles[i]['routeTag']][i] = L.marker([vehicles[i]['lat'], vehicles[i]['lon']], {
					icon: L.icon({
						iconUrl: 'https://rutgers.antsar-static.com/img/bus.png',
						iconSize: [21,25],
						iconAnchor: [10,12],
					}),
					iconAngle: vehicles[i]['heading'],
				}).bindLabel(label, {
					noHide: true,
					direction: 'right',
					className: 'busLabel',
				}).bindPopup(popup).addTo(window.map);
			} else {
				window.busMarkers[vehicles[i]['routeTag']][i].setLatLng([vehicles[i]['lat'], vehicles[i]['lon']]).setPopupContent(popup).setIconAngle(vehicles[i]['heading']);
			}
			window.busMarkers[vehicles[i]['routeTag']][i].lastUpdated = (new Date).getTime();
			
		}
	}
}

function removeStaleMarkers(staleTime) {
	if (staleTime == undefined) staleTime = 60 * 1000; // 60 seconds = dead bus
	for (i in window.busMarkers) {
		for (j in window.busMarkers[i]) {
			if (window.busMarkers[i][j].lastUpdated < (new Date).getTime() - staleTime) {
				window.map.removeLayer(window.busMarkers[i][j]);
				delete window.busMarkers[i][j];
			}
		}
	}
}

function cleanRouteName(route) {
	if (route == "New Brunsquick 1 Shuttle") route = "NB 1";
	else if (route == "New Brunsquick 2 Shuttle") route = "NB 2";
	return route;
}


function closeEnough(arg1, arg2, decimalPlacesAccuracy) {
	var x = Math.pow(10,decimalPlacesAccuracy);
	return (Math.floor(arg1 * x) / x == Math.floor(arg2 * x) / x);
}

function sameBusStop(pos1, pos2) {
	var acc = 3; // up to 110 meters
	if (closeEnough(pos1['lat'], pos2['lat'], acc) && closeEnough(pos1['lon'], pos2['lon'], acc)) return true;
	return false;
}

function ol( object ) {
    var length = 0;
    for( var key in object ) {
        if( object.hasOwnProperty(key) ) {
            ++length;
        }
    }
    return length;
};
